package com.netsitech.caricese.paper.token;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Token
{
	//Formato stringa: 0015;6230;M1245;SC;2013-06-01;15:30:00
	
	static final SimpleDateFormat DATETIMEFMT = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	static final SimpleDateFormat DATEFMT = new SimpleDateFormat("yyyy-MM-dd");
	static final SimpleDateFormat TIMEFMT = new SimpleDateFormat("HH:mm:ss");
	
	public static enum Profile
	{
		DP, //Dipendenza
		SC	//SedeCentrale
	}
	
	Integer 	filiale;
	Integer		codAzienda;
	String 		matricola;
	Profile		profilo;
	Date   		creationDate;
	
	private Token()
	{						
	}
	
	public Token(String values)
	{
		try
		{
			setValues(values);
		}
		catch (ParseException e)
		{
			throw new RuntimeException("Token '"+values+"' non valido", e);
		}				
	}

	public Token(int filiale, int codAzienda, String matricola, Profile profilo)
	{				
		setValues(Integer.valueOf(filiale), Integer.valueOf(codAzienda), matricola, profilo, new Date());
	}

	public Token(int filiale, int codAzienda, String matricola, Profile profilo, Date date)
	{		
		setValues(Integer.valueOf(filiale), Integer.valueOf(codAzienda), matricola, profilo, date);
	}
	
	public Token(Integer filiale, Integer codAzienda, String matricola, Profile profilo)
	{		
		setValues(filiale, codAzienda, matricola, profilo, new Date());
	}
	
	public Integer getFiliale()
	{
		return filiale;
	}

	public void setFiliale(Integer filiale)
	{
		this.filiale = filiale;
	}

	public Integer getCodAzienda()
	{
		return codAzienda;
	}

	public void setCodAzienda(Integer codAzienda)
	{
		this.codAzienda = codAzienda;
	}

	public String getMatricola()
	{
		return matricola;
	}

	public void setMatricola(String matricola)
	{
		this.matricola = matricola;
	}

	public Profile getProfilo()
	{
		return profilo;
	}

	public void setProfilo(Profile profilo)
	{
		this.profilo = profilo;
	}

	public Date getCreationDate()
	{
		return creationDate;
	}

	public void setCreationDate(Date creationDate)
	{
		this.creationDate = creationDate;
	}
				
	public static Token valueOf(String str) throws IllegalArgumentException
	{
		try
		{
			Token result = new Token();		
			result.setValues(str);				
			return result;
		}
		catch (ParseException e)
		{
			throw new RuntimeException("Token '"+str+"' non valido", e);
		}
	}
	
	public void setValues(String values) throws ParseException
	{
		String[] tokens = values.split(";");
		
		if (tokens.length != 6)
			throw new ParseException("Formato non corretto", 0);

		String filiale1 = tokens[0]; 
		String azienda1 = tokens[1];
		String matricola1 = tokens[2];
		String profilo1 = tokens[3];
		String creationDateTime = tokens[4] + " " + tokens[5];
		
		if (filiale1 == null    || filiale1.length() != 4 )    
			throw new ParseException("Filiale non corretta", 0);
		
		if (azienda1 == null || azienda1.length() != 4) 
			throw new ParseException("CodAzienda non corretta", 0);
		
		if (matricola1 == null  || matricola1.length() > 5 )   
			throw new ParseException("Matricola non corretta", 0);
		 
		Integer valFiliale;
		Integer valAzienda;		
		Profile valProfilo;
		Date    valCreationDateTime;
		
		try
		{
			valFiliale = Integer.valueOf(filiale1);
		}
		catch (Exception e)
		{
			throw new ParseException("Filiale '"+filiale1+"' corretta", 0);
		}
		try
		{
			valAzienda = Integer.valueOf(azienda1);
		}
		catch (Exception e)
		{
			throw new ParseException("Codice azienda '"+azienda1+"' non valido", 0);
		}		
		try
		{
			valProfilo = Profile.valueOf(profilo1);
		}
		catch (Exception e)
		{
			throw new ParseException("Profilo '"+profilo1+"' non valido", 0);
		}
		try
		{
			valCreationDateTime = DATETIMEFMT.parse(creationDateTime);
		}
		catch (Exception e)
		{
			throw new ParseException("Ora creazione '"+creationDateTime+"' non validi", 0);
		}		
			
		setValues(valFiliale, valAzienda, matricola1, valProfilo, valCreationDateTime);
	}	
	
	public void setValues(Integer filiale, Integer codAzienda, String matricola, Profile profilo, Date creationDate) throws IllegalArgumentException
	{		
		this.filiale = filiale;
		this.codAzienda = codAzienda;
		this.matricola = matricola;
		this.profilo = profilo;
		this.creationDate = creationDate;
	}
	
	static String format(Integer val)
	{
		return String.format("%04d", val);
	}
	
	@Override
	public String toString()
	{		
		return 	format(this.filiale) + ";" +
				format(this.codAzienda) + ";" +
				this.matricola + ";" +
				this.profilo + ";" +
				DATEFMT.format(this.creationDate) + ";" +
				TIMEFMT.format(this.creationDate);
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (obj instanceof Token)
		{
			Token b = (Token)obj;
				
			return 			
				this.creationDate.equals(b.getCreationDate()) &&
				this.profilo.equals(b.getProfilo()) &&
				this.matricola.equals(b.getMatricola()) &&
				this.codAzienda.equals(b.getCodAzienda()) &&
				this.filiale.equals(b.getFiliale());
		}
		return false;
	}
		
}










