package com.netsitech.caricese.paper.token;

import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;

public class TokenFunctions
{
	public static String encode(Token token, RSAPublicKey pubkey) throws Exception
	{
		return encode(token, pubkey.getPublicExponent(), pubkey.getModulus());
	}

	public static String encode(Token token, BigInteger e, BigInteger n) throws Exception
	{
		try
		{		
			String     info 	  = token.toString();
			
			String     hash		  = calcHash(info);
			
			String     tocriptstr = info + ";" + hash;			
			byte[]	   tocriptbytes = tocriptstr.getBytes();
			
			BigInteger tocriptint = new BigInteger(tocriptbytes);
			
			BigInteger cryptedInt = RSA.encrypt(tocriptint, e, n, "2");
			return cryptedInt.toString();
		}
		catch (Exception ex)
		{
			throw new Exception("Criptazione token fallita", ex);
		}
	}

	public static Token decode(String cryptedToken, RSAPrivateKey privkey) throws Exception
	{
		return decode(cryptedToken,privkey.getPrivateExponent(), privkey.getModulus());
	}
	
	public static Token decode(String cryptedToken, BigInteger d, BigInteger n) throws Exception
	{		
		try
		{
			BigInteger cryptedInt 		= new BigInteger(cryptedToken, 10);		
			BigInteger decryptedInt 	= RSA.decrypt(cryptedInt, d, n, "2");
			byte[]	   decryptedBytes 	= decryptedInt.toByteArray();			
			String 	   decryptedStr 	= new String(decryptedBytes);
				
			int    separatorPos = decryptedStr.lastIndexOf(';'); 		
			if (separatorPos < 0)
				throw new Exception("Valori decriptati non corretti");
			
			String info 		= decryptedStr.substring(0, separatorPos);
			String hash			= decryptedStr.substring(separatorPos + 1);						
			String newHash	    = calcHash(info);
			
			if (!newHash.equals(hash))
				throw new Exception("Hash '"+newHash+"' non corrisponde ai dati");

			return Token.valueOf(info);
		}
		catch (Exception e)
		{
			throw new Exception("Decodifica token fallita", e);
		}
	}
	
	private static String calcHash(String stringa) throws NoSuchAlgorithmException
	{
		java.security.MessageDigest md = java.security.MessageDigest.getInstance("SHA");
		///java.security.MessageDigest md2 = java.security.MessageDigest.getInstance("SHA-1");
	//	System.out.println(md.toString());
//		System.out.println(md2.toString());		
		md.update(stringa.getBytes());
		byte[] digest = md.digest();		
		return new String(org.apache.commons.codec.binary.Base64.encodeBase64(digest));
	}	
	
}
